/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.reporters.pdd;

// REMPL API classes
import com.rempl.api.Element;
import com.rempl.api.Logger;
import com.rempl.api.Model;
import com.rempl.api.Rempler;
import com.rempl.api.Reporter;

// utils
import com.rempl.utils.VelocityWriter;

// file management
import java.io.File;

// collection management
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

// files manipulation
import org.apache.commons.io.FileUtils;

/**
 * Reporter of PDD puzzles.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: PDDReporter.java 795 2011-03-19 10:17:23Z yegor256@yahoo.com $
 */
public final class PDDReporter implements Reporter {

    /**
     * The {@link Rempler} we're working for.
     * @see #setRempler(Rempler)
     */
    private Rempler rempler;

    /**
     * Public ctor.
     */
    public PDDReporter() {
        this.rempler = null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setRempler(final Rempler rpr) {
        this.rempler = rpr;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void report(final Model model, final File dir)
        throws java.io.IOException {
        if (Logger.isTraceEnabled(this)) {
            Logger.trace(this, "#report(model, dir)");
        }
        final String source =
            this.rempler.configuration().get("/*/sources/directory/text()");
        final Collection<Element> puzzles =
            model.query("context Puzzle inv: true");
        final Map<String, Ticket> tickets =
            new HashMap<String, Ticket>();
        for (Element puzzle : puzzles) {
            final String name = puzzle.get("ticket");
            if (!tickets.containsKey(name)) {
                tickets.put(name, new Ticket(name));
            }
            tickets.get(name).append(puzzle);
        }
        final Map<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("tickets", tickets);
        ctx.put("source", source);
        final String html = VelocityWriter.INSTANCE
            .write("html/index.html", ctx);
        final File index = new File(dir + "/index.html");
        FileUtils.writeStringToFile(index, html);
        if (Logger.isDebugEnabled(this)) {
            Logger.debug(
                this,
                "#report(%s, '%s'): %d tickets, %d bytes written to %s",
                model.getClass().getName(),
                dir.getPath(),
                tickets.size(),
                index.length(),
                index.getPath()
            );
        }
    }

}
